@@ -18,6 +18,7 @@
  *   Free Software Foundation, Inc.,                                       *
  *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.          *
  ***************************************************************************/
+#include <QNetworkInterface>
 #include <klocale.h>
 #include <kio/job.h>
 #include <kmessagebox.h>
@@ -29,7 +30,7 @@
 #include <solid/powermanagement.h>
 
 
-
+#include <dbus/dbus.h>
 #include <interfaces/guiinterface.h>
 #include <interfaces/functions.h>
 #include <interfaces/torrentfileinterface.h>
@@ -64,6 +65,7 @@
 #include "gui.h"
 #include "torrentmigratordlg.h"
 
+
 using namespace bt;
 
 namespace kt
@@ -74,8 +76,13 @@ namespace kt
 	{
 		UpdateCurrentTime();
 		qman = new QueueManager();
-		connect(qman, SIGNAL(lowDiskSpace(bt::TorrentInterface*,bool)), this, SLOT(onLowDiskSpace(bt::TorrentInterface*,bool)));
-
+		connect(qman, SIGNAL(lowDiskSpace(bt::TorrentInterface*,bool)), 
+				this, SLOT(onLowDiskSpace(bt::TorrentInterface*,bool)));
+		connect(qman, SIGNAL(queuingNotPossible( bt::TorrentInterface* )),
+				this, SLOT(enqueueTorrentOverMaxRatio( bt::TorrentInterface* )));
+		connect(qman, SIGNAL(lowDiskSpace(bt::TorrentInterface*, bool)),
+				this, SLOT(onLowDiskSpace(bt::TorrentInterface*, bool)));
+		
 		data_dir = Settings::tempDir().path();
 		bool dd_not_exist = !bt::Exists(data_dir);
 		if (data_dir == QString::null || dd_not_exist)
@@ -102,6 +109,19 @@ namespace kt
 			port = 6881;
 			Settings::setPort(6881);
 		}
+		
+		// Make sure network interface is set properly before server is initialized
+		if (Settings::networkInterface() != 0)
+		{
+			QList<QNetworkInterface> iface_list = QNetworkInterface::allInterfaces();
+			int iface = Settings::networkInterface();
+			if (iface > iface_list.count())
+				SetNetworkInterface(QString::null);
+			else
+				SetNetworkInterface(iface_list[iface - 1].name());
+		}
+		
+		
 		Uint16 i = 0;
 		do
 		{
@@ -129,7 +149,7 @@ namespace kt
 		
 		pman = new kt::PluginManager(this,gui);
 		gman = new kt::GroupManager();
-		applySettings(true);
+		applySettings();
 		gman->loadGroups();
 		connect(qman,SIGNAL(queueOrdered()),this,SLOT(startUpdateTimer()));
 		connect(qman,SIGNAL(pauseStateChanged(bool)),gui,SLOT(onPausedStateChanged(bool)));
@@ -146,12 +166,6 @@ namespace kt
 	}
 
 	void Core::applySettings()
-	{
-		applySettings(false);
-		settingsChanged();
-	}
-
-	void Core::applySettings(bool change_port)
 	{
 		ApplySettings();
 		setMaxDownloads(Settings::maxDownloads());
@@ -163,11 +177,11 @@ namespace kt
 			tmp = kt::DataDir();
 		
 		changeDataDir(tmp);
-		if (change_port)
-			changePort(Settings::port());
-		        
+		changePort(Settings::port());
+		
 		//update QM
 		getQueueManager()->orderQueue();
+		settingsChanged();
 	}
 
 	void Core::loadPlugins()
@@ -181,13 +195,15 @@ namespace kt
 		bool start_torrent = false;
 		bool skip_check = false;
 
-		connectSignals(tc);
-		qman->append(tc);
 		if (!silently)
 		{
 			if (!gui->selectFiles(tc,&user,&start_torrent,group,&skip_check))
 			{
-				remove(tc,false);
+				// Cleanup tor dir
+				QString dir = tc->getTorDir();
+				if (bt::Exists(dir))
+					bt::Delete(dir,true);
+				delete tc;
 				return false;
 			}
 		}
@@ -216,6 +232,9 @@ namespace kt
 				}
 			}
 		}
+		
+		connectSignals(tc);
+		qman->append(tc);
 	
 		try
 		{
@@ -223,7 +242,9 @@ namespace kt
 		}
 		catch (bt::Error & err)
 		{
-			gui->errorMsg(err.toString());
+			if (!silently)
+				gui->errorMsg(err.toString());
+			Out(SYS_GEN|LOG_IMPORTANT) << err.toString() << endl;
 			remove(tc,false);
 			return false;
 		}
@@ -231,15 +252,17 @@ namespace kt
 		if (tc->hasExistingFiles())
 		{
 			if (!skip_check)
-				gui->dataScan(tc,true,true,QString::null);
+				gui->dataScan(tc,false,true,QString::null);
 			else
 				tc->markExistingFilesAsDownloaded();
 		}
 			
 		tc->setPreallocateDiskSpace(true);
 
-		if(Settings::maxRatio()>0)
-			tc->setMaxShareRatio(Settings::maxRatio()); 
+		if (Settings::maxRatio() > 0)
+			tc->setMaxShareRatio(Settings::maxRatio());
+		if (Settings::maxSeedTime() > 0)
+			tc->setMaxSeedTime(Settings::maxSeedTime());
 		
 		torrentAdded(tc);
 		qman->torrentAdded(tc,user,start_torrent);
@@ -271,6 +294,7 @@ namespace kt
 			tc = new TorrentControl();
 			tc->init(qman, data, tdir, dir, 
 					 Settings::useSaveDir() ? Settings::saveDir().path() : QString());
+			tc->setLoadUrl(url);
 			
 			if(!init(tc,group,silently))
 				loadingFinished(url, false, true);
@@ -311,6 +335,7 @@ namespace kt
 			tc = new TorrentControl();
 			tc->init(qman, target, tdir, dir, 
 				 Settings::useSaveDir() ? Settings::saveDir().path() : QString());
+			tc->setLoadUrl(KUrl(target));
 			
 			init(tc,group,silently);
 			startUpdateTimer();
@@ -468,7 +493,48 @@ namespace kt
 				add_to_groups.insert(j,group);
 		}
 	}
+	
+	void Core::load(const QByteArray & data,const KUrl& url,const QString & group,const QString & savedir)
+	{
+		QString dir;
+		if (savedir.isEmpty() || !bt::Exists(savedir))
+		{
+			dir = Settings::saveDir().path();
+			if (!Settings::useSaveDir()  || dir.isNull())
+				dir =  QDir::homePath();
+		}
+		else
+			dir = savedir;
+		
+		if (dir != QString::null && load(data,dir,group,false,url))
+			loadingFinished(url,true,false);
+		else
+			loadingFinished(url,false,true);
+	}
+	
+	void Core::loadSilently(const QByteArray & data,const KUrl& url,const QString & group,const QString & savedir)
+	{
+		QString dir;
+		if (savedir.isEmpty() || !bt::Exists(savedir))
+		{
+			dir = Settings::saveDir().path();
+			if (!Settings::useSaveDir())
+			{
+				Out(SYS_GEN|LOG_NOTICE) << "Cannot load " << url.prettyUrl() << " silently, default save location not set !" << endl;
+				Out(SYS_GEN|LOG_NOTICE) << "Using home directory instead !" << endl;
+				dir = QDir::homePath();
+			}
+		}
+		else
+			dir = savedir;
+		
+		if (dir != QString::null && load(data,dir,group,true,url))
+			loadingFinished(url,true,false);
+		else
+			loadingFinished(url,false,true);
+	}
 
+	/*
 	void Core::loadSilentlyDir(const KUrl& url, const KUrl& savedir)
 	{
 		if (url.isLocalFile())
@@ -500,7 +566,8 @@ namespace kt
 			connect(j,SIGNAL(result(KJob*)),this,SLOT(downloadFinishedSilently( KJob* )));
 		}
 	}
-
+*/
+	
 	void Core::start(bt::TorrentInterface* tc)
 	{
 		TorrentStartResponse reason = qman->start(tc);
@@ -579,7 +646,7 @@ namespace kt
 		QStringList filters;
 		filters << "tor*";
 		QStringList sl = dir.entryList(filters,QDir::Dirs);
-		for (Uint32 i = 0;i < sl.count();i++)
+		for (int i = 0;i < sl.count();i++)
 		{
 			QString idir = data_dir + sl.at(i);
 			if (!idir.endsWith(DirSeparator()))
@@ -598,7 +665,7 @@ namespace kt
 			const bt::TorrentStats & s = tc->getStats();
 			removed_bytes_up += s.session_bytes_uploaded;
 			removed_bytes_down += s.session_bytes_downloaded;
-			stop(tc);
+			stop(tc,true);
 
 			QString dir = tc->getTorDir();
 			
@@ -613,9 +680,9 @@ namespace kt
 			}
 			
 			torrentRemoved(tc);
-			qman->torrentRemoved(tc);
 			gman->torrentRemoved(tc);
-
+			qman->torrentRemoved(tc);
+			gui->updateActions();
 			bt::Delete(dir,false);
 		}
 		catch (Error & e)
@@ -672,8 +739,8 @@ namespace kt
 		else
 			delete job;
 		
-		qman->clear();
 		pman->unloadAll();
+		qman->clear();
 	}
 
 	bool Core::changeDataDir(const QString & new_dir)
@@ -893,15 +960,9 @@ namespace kt
 
 	bool Core::changePort(Uint16 port)
 	{
-		if (qman->count() == 0)
-		{
-			Globals::instance().getServer().changePort(port);
-			return true;
-		}
-		else
-		{
-			return false;
-		}
+		bt::Server & srv = Globals::instance().getServer();
+		srv.changePort(port);
+		return srv.isOK();
 	}
 
 	void Core::slotStoppedByError(bt::TorrentInterface* tc, QString msg)
@@ -1066,13 +1127,11 @@ namespace kt
 		connect(tc,SIGNAL(aboutToBeStarted( bt::TorrentInterface*,bool & )),
 				this, SLOT(aboutToBeStarted( bt::TorrentInterface*,bool & )));
 		connect(tc,SIGNAL(corruptedDataFound( bt::TorrentInterface* )),
-				this, SLOT(emitCorruptedData( bt::TorrentInterface* )));
-		connect(qman, SIGNAL(queuingNotPossible( bt::TorrentInterface* )),
-				this, SLOT(enqueueTorrentOverMaxRatio( bt::TorrentInterface* )));
-		connect(qman, SIGNAL(lowDiskSpace(bt::TorrentInterface*, bool)),
-				this, SLOT(onLowDiskSpace(bt::TorrentInterface*, bool)));
+				this, SLOT(emitCorruptedData( bt::TorrentInterface* )));	
 		connect(tc, SIGNAL(needDataCheck(bt::TorrentInterface*)),
 				this, SLOT(autoCheckData(bt::TorrentInterface*)));
+		connect(tc,SIGNAL(statusChanged(bt::TorrentInterface*)),
+				this,SLOT(onStatusChanged(bt::TorrentInterface*)));
 	}
 
 	float Core::getGlobalMaxShareRatio() const
@@ -1146,6 +1205,17 @@ namespace kt
 			KMessageBox::information(gui,i18n("No torrents from the KDE3 version were found !"));
 		}
 	}
+	
+	DBus* Core::getExternalInterface()
+	{
+		return gui->getDBusInterface();
+	}
+	
+	void Core::onStatusChanged(bt::TorrentInterface* tc)
+	{
+		Q_UNUSED(tc);
+		gui->updateActions();
+	}
 }
 
 #include "core.moc"
