@@ -60,41 +60,44 @@
 #include "groupview.h"
 #include "scandlg.h"
 #include "trayicon.h"
-#include "dbus.h"
+#include "dbus/dbus.h"
 #include "pastedialog.h"
 #include "ipfilterwidget.h"
 #include "torrentcreatordlg.h"
 #include "importdialog.h"
 #include "speedlimitsdlg.h"
 #include "queuemanagerwidget.h"
+#include <util/timer.h>
 
 
 namespace kt
 {
 	GUI::GUI() : core(0),pref_dlg(0)
 	{
+		//Marker markk("GUI::GUI()");
 		core = new Core(this);
 		tray_icon = new TrayIcon(core,this);
-		setupActions();
-		setActionsEnabled((ActionEnableFlags)0);
 		view_man = new ViewManager(core->getGroupManager()->allGroup(),this,core);
-		connect(view_man,SIGNAL(enableActions(ActionEnableFlags)),this,SLOT(setActionsEnabled(ActionEnableFlags)));
+		setupActions();
+		view_man->setupActions();
 		
 		group_view = new GroupView(core->getGroupManager(),view_man,this);
-		addToolWidget(group_view,"application-x-bittorrent",i18n("Groups"),DOCK_LEFT);
-		connect(group_view,SIGNAL(openNewTab(kt::Group*)),this,SLOT(openView(kt::Group*)));
+		addToolWidget(group_view,"application-x-bittorrent",i18n("Groups"),i18n("Widget to manage torrent groups"),DOCK_LEFT);
+		connect(group_view,SIGNAL(openNewTab(kt::Group*)),this,SLOT(openNewView(kt::Group*)));
 
 		qm = new QueueManagerWidget(core->getQueueManager(),this);
 		connect(core,SIGNAL(torrentAdded(bt::TorrentInterface*)),qm,SLOT(onTorrentAdded(bt::TorrentInterface*)));
 		connect(core,SIGNAL(torrentRemoved(bt::TorrentInterface*)),qm,SLOT(onTorrentRemoved(bt::TorrentInterface*)));
-		addToolWidget(qm,"kt-queue-manager",i18n("Queue Manager"),DOCK_BOTTOM);
+		addToolWidget(qm,"kt-queue-manager",i18n("Queue Manager"),i18n("Widget to manage the torrent queue"),DOCK_BOTTOM);
 		
 		createGUI("ktorrentui.rc");
 		
 		status_bar = new kt::StatusBar(this);
 		setStatusBar(status_bar);
 
+		//Marker mark("core->loadTorrents()");
 		core->loadTorrents();
+		//mark.update();
 		connect(&timer,SIGNAL(timeout()),this,SLOT(update()));
 		timer.start(Settings::guiUpdateInterval());
 		
@@ -108,7 +111,6 @@ namespace kt
 
 		applySettings();
 		connect(core,SIGNAL(settingsChanged()),this,SLOT(applySettings()));
-
 		currentTabPageChanged(currentTabPage());
 
 		if (Settings::showSystemTrayIcon())
@@ -119,20 +121,23 @@ namespace kt
 		else
 			tray_icon->hide();		
 
-		dbus_iface = new DBus(this,core);
-		
+		dbus_iface = new DBus(this,core,this);
+		view_man->loadState(KGlobal::config());
 		core->loadPlugins();
 		loadState(KGlobal::config());
 		notifyViewListeners(view_man->getCurrentTorrent());
+		//markk.update();
+		updateActions();
 	}
 
 	GUI:: ~GUI()
 	{
+		delete core;
 	}
 
-	void GUI::addTabPage(QWidget* page,const QString & icon,const QString & caption,CloseTabListener* ctl)
+	void GUI::addTabPage(QWidget* page,const QString & icon,const QString & caption,const QString & tooltip,CloseTabListener* ctl)
 	{
-		addTab(page,caption,icon);
+		addTab(page,caption,icon,CENTER,tooltip);
 		close_tab_map[page] = ctl;	
 		currentTabPageChanged(currentTabPage());
 	}
@@ -173,7 +178,7 @@ namespace kt
 		guiFactory()->removeClient(p);
 	}
 
-	void GUI::addToolWidget(QWidget* w,const QString & icon,const QString & caption,ToolDock dock)
+	void GUI::addToolWidget(QWidget* w,const QString & icon,const QString & caption,const QString & tooltip,ToolDock dock)
 	{
 		ideal::MainWindow::TabPosition pos = LEFT;
 		switch (dock)
@@ -188,7 +193,7 @@ namespace kt
 				pos = RIGHT;
 				break;
 		}
-		addTab(w,caption,icon,pos);
+		addTab(w,caption,icon,pos,tooltip);
 	}
 	
 	void GUI::removeToolWidget(QWidget* w)
@@ -203,9 +208,16 @@ namespace kt
 		return view_man->getCurrentTorrent();
 	}
 	
+	bt::TorrentInterface* GUI::getCurrentTorrent()
+	{
+		return view_man->getCurrentTorrent();
+	}
+	
 	void GUI::dataScan(bt::TorrentInterface* tc,bool auto_import,bool silently,const QString & dlg_caption)
 	{
 		ScanDlg* dlg = new ScanDlg(core,auto_import,this);
+		if (!dlg_caption.isEmpty())
+			dlg->setWindowTitle(dlg_caption);
 		dlg->show();
 		dlg->execute(tc,silently);
 		core->startUpdateTimer(); // make sure update timer is running
@@ -294,31 +306,12 @@ namespace kt
 			}
 		}
 	}
-
-	void GUI::startTorrent()
-	{
-		view_man->startTorrents();
-	}
-
-	void GUI::stopTorrent()
-	{
-		view_man->stopTorrents();
-	}
-
-	void GUI::removeTorrent()
-	{
-		view_man->removeTorrents();
-	}
-
-	void GUI::queueTorrent()
-	{
-		view_man->queueTorrents();
-	}
 	
 	void GUI::pauseQueue(bool pause)
 	{
-		Out(SYS_GEN|LOG_DEBUG) << "pauseQUeue " << pause << endl;
+		Out(SYS_GEN|LOG_NOTICE) << "Setting paused state to " << pause << endl;
 		core->setPausedState(pause);
+		view_man->updateActions();
 	}
 	
 	void GUI::onPausedStateChanged(bool paused)
@@ -346,11 +339,6 @@ namespace kt
 		core->stopAll(3);
 	}
 
-	void GUI::checkData()
-	{
-		view_man->checkData();
-	}
-
 	void GUI::pasteURL()
 	{
 		PasteDialog dlg(core, this);
@@ -359,6 +347,9 @@ namespace kt
 	
 	void GUI::paste()
 	{
+		if (!paste_action->isEnabled())
+			return;
+		
 		QClipboard *cb = QApplication::clipboard();
 		QString text = cb->text(QClipboard::Clipboard);
 		if (text.length() == 0)
@@ -430,10 +421,14 @@ namespace kt
 	{
 		KActionCollection* ac = actionCollection();
 		KAction* new_action = KStandardAction::openNew(this,SLOT(createTorrent()),ac);
+		new_action->setToolTip(i18n("Create a new torrent"));
 		KAction* open_action = KStandardAction::open(this, SLOT(openTorrent()),ac);
-		KStandardAction::paste(this,SLOT(paste()),ac);
+		open_action->setToolTip(i18n("Open a torrent"));
+		paste_action = KStandardAction::paste(this,SLOT(paste()),ac);
+		KStandardAction::selectAll(view_man,SLOT(selectAll()),ac);
 		
 		open_silently_action = new KAction(KIcon(open_action->icon()),i18n("Open Silently"),this);
+		open_silently_action->setToolTip(i18n("Open a torrent without asking any questions"));
 		connect(open_silently_action,SIGNAL(triggered()),this,SLOT(openTorrentSilently()));
 		ac->addAction("file_open_silently",open_silently_action);
 				
@@ -448,59 +443,84 @@ namespace kt
 		KStandardAction::configureNotifications(this,SLOT(configureNotifications()),ac);
 
 		start_action = new KAction(KIcon("kt-start"),i18n("Start"), this);
-		connect(start_action,SIGNAL(triggered()),this,SLOT(startTorrent()));
+		start_action->setToolTip(i18n("Start all selected torrents in the current tab"));
+		start_action->setShortcut(KShortcut(Qt::CTRL + Qt::Key_S));
+		connect(start_action,SIGNAL(triggered()),view_man,SLOT(startTorrents()));
 		ac->addAction("start",start_action);
 
 		stop_action = new KAction(KIcon("kt-stop"),i18n("Stop"),this);
-		connect(stop_action,SIGNAL(triggered()),this,SLOT(stopTorrent()));
+		stop_action->setToolTip(i18n("Stop all selected torrents in the current tab"));
+		stop_action->setShortcut(KShortcut(Qt::CTRL + Qt::Key_H));
+		connect(stop_action,SIGNAL(triggered()),view_man,SLOT(stopTorrents()));
 		ac->addAction("stop",stop_action);
 
 		remove_action = new KAction(KIcon("kt-remove"),i18n("Remove"),this);
-		connect(remove_action,SIGNAL(triggered()),this,SLOT(removeTorrent()));
+		remove_action->setToolTip(i18n("Remove all selected torrents in the current tab"));
+		remove_action->setShortcut(KShortcut(Qt::Key_Delete));
+		connect(remove_action,SIGNAL(triggered()),view_man,SLOT(removeTorrents()));
 		ac->addAction("remove",remove_action);
 
 		start_all_cv_action = new KAction(KIcon("kt-start-all"),i18n("Start All"),this);
+		start_all_cv_action->setToolTip(i18n("Start all torrents in the current tab"));
+		start_all_cv_action->setShortcut(KShortcut(Qt::SHIFT + Qt::Key_S));
 		connect(start_all_cv_action,SIGNAL(triggered()),this,SLOT(startAllTorrentsCV()));
 		ac->addAction("start_all",start_all_cv_action);
 		
 		start_all_action = new KAction(KIcon("kt-start-all"),i18n("Start All"),this);
+		start_all_action->setToolTip(i18n("Start all torrents"));
 		connect(start_all_action,SIGNAL(triggered()),this,SLOT(startAllTorrents()));
 		
 		stop_all_cv_action = new KAction(KIcon("kt-stop-all"),i18n("Stop All"),this);
+		stop_all_cv_action->setToolTip(i18n("Stop all torrents in the current tab"));
+		stop_all_cv_action->setShortcut(KShortcut(Qt::SHIFT + Qt::Key_H));
 		connect(stop_all_cv_action,SIGNAL(triggered()),this,SLOT(stopAllTorrentsCV()));
 		ac->addAction("stop_all",stop_all_cv_action);
 		
 		stop_all_action = new KAction(KIcon("kt-stop-all"),i18n("Stop All"),this);
+		stop_all_action->setToolTip(i18n("Stop all torrents"));
 		connect(stop_all_action,SIGNAL(triggered()),this,SLOT(stopAllTorrents()));
 		
 		paste_url_action = new KAction(KIcon(open_action->icon()),i18n("Open URL"),this);
+		paste_url_action->setToolTip(i18n("Open an URL which points to a torrent"));
+		paste_url_action->setShortcut(KShortcut(Qt::CTRL + Qt::Key_P));
 		connect(paste_url_action,SIGNAL(triggered()),this,SLOT(pasteURL()));
 		ac->addAction("paste_url",paste_url_action);
 
 		queue_action = new KAction(KIcon("view-choose"),i18n("Enqueue/Dequeue"),this);
-		connect(queue_action,SIGNAL(triggered()),this,SLOT(queueTorrent()));
+		queue_action->setToolTip(i18n("Enqueue or dequeue all selected torrents in the current tab"));
+		queue_action->setShortcut(KShortcut(Qt::CTRL + Qt::Key_D));
+		connect(queue_action,SIGNAL(triggered()),view_man,SLOT(queueTorrents()));
 		ac->addAction("queue_action",queue_action);
 		
-		queue_pause_action = new KToggleAction(KIcon("kt-pause"),i18n("Pause"),this);
+		queue_pause_action = new KToggleAction(KIcon("kt-pause"),i18n("Pause KTorrent"),this);
 		queue_pause_action->setToolTip(i18n("Pause all running torrents"));
+		queue_pause_action->setShortcut(KShortcut(Qt::SHIFT + Qt::Key_P));
 		connect(queue_pause_action,SIGNAL(toggled(bool)),this,SLOT(pauseQueue(bool)));
-		queue_pause_action->setCheckedState(KGuiItem(i18n("Resume"),"media-playback-start",i18n("Resume paused torrents")));
+		queue_pause_action->setCheckedState(KGuiItem(i18n("Resume KTorrent"),"media-playback-start",
+											i18n("Resume paused torrents")));
 		ac->addAction("queue_pause",queue_pause_action);
 
 		ipfilter_action = new KAction(KIcon("view-filter"),i18n("IP Filter"),this);
+		ipfilter_action->setToolTip(i18n("Show the list of blocked IP addresses"));
+		ipfilter_action->setShortcut(KShortcut(Qt::CTRL + Qt::Key_I));
 		connect(ipfilter_action,SIGNAL(triggered()),this,SLOT(showIPFilter()));
 		ac->addAction("ipfilter_action",ipfilter_action);
 
 		data_check_action = new KAction(KIcon("kt-check-data"),i18n("Check Data"),this);
-		connect(data_check_action,SIGNAL(triggered()),this,SLOT(checkData()));
+		data_check_action->setToolTip(i18n("Check all the data of a torrent"));
+		data_check_action->setShortcut(KShortcut(Qt::SHIFT + Qt::Key_C));
+		connect(data_check_action,SIGNAL(triggered()),view_man,SLOT(checkData()));
 		ac->addAction("check_data",data_check_action);
 		
 		import_action = new KAction(KIcon("document-import"),i18n("Import Torrent"),this);
+		import_action->setToolTip(i18n("Import a torrent"));
+		import_action->setShortcut(KShortcut(Qt::SHIFT + Qt::Key_I));
 		connect(import_action,SIGNAL(triggered()),this,SLOT(import()));
 		ac->addAction("import",import_action);
 		
 #ifndef Q_WS_WIN
 		import_kde3_torrents_action = new KAction(KIcon("document-import"),i18n("Import KDE3 Torrents"),this);
+		import_kde3_torrents_action->setToolTip(i18n("Import all torrents from the KDE3 version of KTorrent"));
 		connect(import_kde3_torrents_action,SIGNAL(triggered()),core,SLOT(importKDE3Torrents()));
 		ac->addAction("import_kde3_torrents",import_kde3_torrents_action);
 #else
@@ -508,6 +528,7 @@ namespace kt
 #endif
 		
 		speed_limits_action = new KAction(KIcon("kt-speed-limits"),i18n("Speed Limits"),this);
+		speed_limits_action->setToolTip(i18n("Set the speed limits of individual torrents"));
 		connect(speed_limits_action,SIGNAL(triggered()),this,SLOT(speedLimits()));
 		speed_limits_action->setShortcut(KShortcut(Qt::CTRL + Qt::Key_L));
 		ac->addAction("speed_limits",speed_limits_action);
@@ -528,7 +549,6 @@ namespace kt
 		m->addSeparator();
 		m->addAction(paste_url_action);
 		m->addAction(open_action);
-		m->addAction(new_action);
 		m->addSeparator();
 		m->addAction(pref_action);
 		m->addAction(show_kt_action);
@@ -545,11 +565,9 @@ namespace kt
 		status_bar->updateTransfer(stats.bytes_uploaded,stats.bytes_downloaded);
 		status_bar->updateDHTStatus(Globals::instance().getDHT().isRunning(),Globals::instance().getDHT().getStats());
 
-		tray_icon->updateStats(stats,Settings::showSpeedBarInTrayIcon(),Settings::downloadBandwidth(), Settings::uploadBandwidth());
+		tray_icon->updateStats(stats);
 		core->updateGuiPlugins();
 		
-		start_all_action->setEnabled(core->getNumTorrentsNotRunning() > 0);
-		stop_all_action->setEnabled(core->getNumTorrentsRunning() > 0);
 		if (qm->isVisible())
 			qm->update();
 	}
@@ -580,17 +598,27 @@ namespace kt
 
 	void GUI::newView()
 	{
-		openView(core->getGroupManager()->allGroup());
+		newView(core->getGroupManager()->allGroup());
 	}
 
-	void GUI::openView(kt::Group* g)
+	View* GUI::newView(kt::Group* g)
 	{
 		View* view = view_man->newView(core,this);
 		view->setGroup(g);
-		addTabPage(view,g->groupIconName(),view->caption(),view_man);
+		addTabPage(view,g->groupIconName(),view->caption(false),QString(),view_man);
+		changeTabToolTip(view,view->caption(true));
+		connect(view,SIGNAL(editingItem(bool)),this,SLOT(setPasteDisabled(bool)));
+		return view;
+	}
+			
+	void GUI::openNewView(kt::Group* g)
+	{
+		View* v = newView(g);
+		v->setupDefaultColumns();
 	}
 	
-	void GUI::openView(const QString & group_name)
+	
+	void GUI::openView(const QString & group_name,bool starting_up)
 	{
 		Group* g = core->getGroupManager()->find(group_name);
 		if (!g)
@@ -600,7 +628,9 @@ namespace kt
 				g = core->getGroupManager()->allGroup();
 		}
 
-		openView(g);
+		View* v = newView(g);
+		if (!starting_up) // if it is a new view, setup the default columns based upon the group
+			v->setupDefaultColumns();
 	}
 
 	void GUI::currentTabPageChanged(QWidget* page)
@@ -615,13 +645,14 @@ namespace kt
 	
 	void GUI::speedLimits()
 	{
-		SpeedLimitsDlg dlg(0,core,this);
+		QList<bt::TorrentInterface*> sel;
+		view_man->getSelection(sel);
+		SpeedLimitsDlg dlg(sel.count() > 0 ? sel.front() : 0,core,this);
 		dlg.exec();
 	}
 	
 	void GUI::loadState(KSharedConfigPtr cfg)
 	{
-		view_man->loadState(cfg);
 		group_view->loadState(cfg);
 		qm->loadState(cfg);
 		ideal::MainWindow::loadState(cfg);
@@ -699,15 +730,14 @@ namespace kt
 		}
 		return true;
 	}
-
-	void GUI::setActionsEnabled(ActionEnableFlags flags)
+	
+	void GUI::updateActions()
 	{
-		start_action->setEnabled(flags & START);
-		stop_action->setEnabled(flags & STOP);
-		remove_action->setEnabled(flags & REMOVE);
-		start_all_cv_action->setEnabled(flags & START_ALL);
-		stop_all_cv_action->setEnabled(flags & STOP_ALL);
-		queue_pause_action->setEnabled(core->getPausedState() || flags & STOP_ALL);
+		view_man->updateActions();
+		Uint32 nr = core->getNumTorrentsRunning();
+		queue_pause_action->setEnabled(core->getPausedState() || nr > 0);
+		start_all_action->setEnabled(core->getNumTorrentsNotRunning() > 0);
+		stop_all_action->setEnabled(nr > 0);
 	}
 	
 	void GUI::showOrHide()
@@ -739,6 +769,11 @@ namespace kt
 	{
 		return currentTabPage();
 	}
+	
+	void GUI::setPasteDisabled(bool on)
+	{
+		paste_action->setEnabled(!on);
+	}
 }
 
 #include "gui.moc"
