@@ -70,7 +70,7 @@ namespace kt
 		connect(m_core,SIGNAL(canNotLoadSilently(const QString&)),
 				this,SLOT(cannotLoadTorrentSilently(const QString&)));
 		
-		KMenu* m = new KMenu(0);
+		KMenu* m = new KMenu(parent);
 		setContextMenu(m);
 		max_upload_rate = new SetMaxRate(core,SetMaxRate::UPLOAD,m);
 		max_upload_rate->setTitle(i18n("Set max upload speed"));
@@ -84,7 +84,7 @@ namespace kt
 	TrayIcon::~TrayIcon()
 	{}
 
-	void TrayIcon::updateStats(const CurrentStats stats, bool showBars,int downloadBandwidth, int uploadBandwidth )
+	void TrayIcon::updateStats(const CurrentStats & stats)
 	{
 		if (!Settings::showPopups())
 		{
@@ -93,42 +93,13 @@ namespace kt
 		else
 		{
 			QString tip = i18n("<table cellpadding='2' cellspacing='2' align='center'><tr><td><b>Speed:</b></td><td></td></tr><tr><td>Download: <font color='#1c9a1c'>%1</font></td><td>Upload: <font color='#990000'>%2</font></td></tr><tr><td><b>Transfer:</b></td><td></td></tr><tr><td>Download: <font color='#1c9a1c'>%3</font></td><td>Upload: <font color='#990000'>%4</font></td></tr></table>",
-				KBytesPerSecToString((double)stats.download_speed/1024.0),
-				KBytesPerSecToString((double)stats.upload_speed/1024.0),
+				BytesPerSecToString((double)stats.download_speed),
+				BytesPerSecToString((double)stats.upload_speed),
 				BytesToString(stats.bytes_downloaded),
 				BytesToString(stats.bytes_uploaded));
 		
 			setToolTip(tip);
-		}	
-		if(showBars)
-			drawSpeedBar(stats.download_speed/1024,stats.upload_speed/1024, downloadBandwidth, uploadBandwidth);
-	}
-
-	void TrayIcon::drawSpeedBar(int downloadSpeed, int uploadSpeed, int downloadBandwidth, int uploadBandwidth )
-	{
-		/*
-		//check if need repaint
-		if (uploadBandwidth == 0)
-			 uploadBandwidth = 1;
-		if (downloadBandwidth == 0)
-			downloadBandwidth = 1;
-			
-		QRect r = geometry();
-		int DownloadHeight= (downloadSpeed * r.height()) / downloadBandwidth;
-		int UploadHeight= (uploadSpeed * r.height()) / uploadBandwidth;
-		if (previousDownloadHeight == DownloadHeight && previousUploadHeight == UploadHeight)
-			return;
-		
-		repaint ();
-
-		QBrush brushD(green);
-		QBrush brushU(red);
-		paint->fillRect(0,r.height(),2,-DownloadHeight,brushD);
-		paint->fillRect(r.width()-2,r.height(),2,-UploadHeight,brushU);
-
-		previousDownloadHeight=DownloadHeight;
-		previousUploadHeight=UploadHeight;	
-		*/
+		}
 	}
 
 	void TrayIcon::showPassivePopup(const QString & msg,const QString & title)
@@ -152,14 +123,14 @@ namespace kt
 			return;
 		
 		const TorrentStats & s = tc->getStats();
-		double speed_up = (double)s.bytes_uploaded / 1024.0;
-		double speed_down = (double)(s.bytes_downloaded - s.imported_bytes)/ 1024.0;
+		double speed_up = (double)s.bytes_uploaded;
+		double speed_down = (double)(s.bytes_downloaded - s.imported_bytes);
 
 		QString msg = i18n("<b>%1</b> has completed downloading."
 					"<br>Average speed: %2 DL / %3 UL.",
-					s.torrent_name,
-					KBytesPerSecToString(speed_down / tc->getRunningTimeDL()),
-					KBytesPerSecToString(speed_up / tc->getRunningTimeUL()));
+					tc->getDisplayName(),
+					BytesPerSecToString(speed_down / tc->getRunningTimeDL()),
+					BytesPerSecToString(speed_up / tc->getRunningTimeUL()));
 
 		KNotification::event("TorrentFinished",msg,QPixmap(),mwnd);
 	}
@@ -171,14 +142,14 @@ namespace kt
 		
 		const TorrentStats & s = tc->getStats();
 		KLocale* loc = KGlobal::locale();
-		double speed_up = (double)s.bytes_uploaded / 1024.0;
+		double speed_up = (double)s.bytes_uploaded;
 		
 		QString msg = i18n("<b>%1</b> has reached its maximum share ratio of %2 and has been stopped."
 				"<br>Uploaded %3 at an average speed of %4.",
-				s.torrent_name,
+				tc->getDisplayName(),
 				loc->formatNumber(s.max_share_ratio,2),
 				BytesToString(s.bytes_uploaded),
-				KBytesPerSecToString(speed_up / tc->getRunningTimeUL()));
+				BytesPerSecToString(speed_up / tc->getRunningTimeUL()));
 
 		KNotification::event("MaxShareRatioReached",msg,QPixmap(),mwnd);
 	}
@@ -190,14 +161,14 @@ namespace kt
 		
 		const TorrentStats & s = tc->getStats();
 		KLocale* loc = KGlobal::locale();
-		double speed_up = (double)s.bytes_uploaded / 1024.0;
+		double speed_up = (double)s.bytes_uploaded;
 		
 		QString msg = i18n("<b>%1</b> has reached its maximum seed time of %2 hours and has been stopped."
 				"<br>Uploaded %3 at an average speed of %4.",
-				s.torrent_name,
+				tc->getDisplayName(),
 				loc->formatNumber(s.max_seed_time,2),
 				BytesToString(s.bytes_uploaded),
-				KBytesPerSecToString(speed_up / tc->getRunningTimeUL()));
+				BytesPerSecToString(speed_up / tc->getRunningTimeUL()));
 
 		KNotification::event("MaxSeedTimeReached",msg,QPixmap(),mwnd);
 	}
@@ -207,9 +178,8 @@ namespace kt
 		if (!Settings::showPopups())
 			return;
 		
-		const TorrentStats & s = tc->getStats();
 		QString err_msg = i18n("<b>%1</b> has been stopped with the following error: <br>%2",
-					s.torrent_name,msg);
+					tc->getDisplayName(),msg);
 
 		KNotification::event("TorrentStoppedByError",err_msg,QPixmap(),mwnd);
 	}
@@ -219,9 +189,8 @@ namespace kt
 		if (!Settings::showPopups())
 			return;
 		
-		const TorrentStats & s = tc->getStats();
 		QString err_msg = i18n("Corrupted data has been found in the torrent <b>%1</b>"
-				"<br>It would be a good idea to do a data integrity check on the torrent.",s.torrent_name);
+				"<br>It would be a good idea to do a data integrity check on the torrent.",tc->getDisplayName());
 				
 		KNotification::event("CorruptedData",err_msg,QPixmap(),mwnd);
 	}
@@ -239,11 +208,11 @@ namespace kt
 		if (tc->overMaxRatio())
 			msg = i18n("<b>%1</b> has reached its maximum share ratio of %2 and cannot be enqueued. "
 					"<br>Remove the limit manually if you want to continue seeding.",
-				s.torrent_name,loc->formatNumber(s.max_share_ratio,2));
+				tc->getDisplayName(),loc->formatNumber(s.max_share_ratio,2));
 		else
 			msg = i18n("<b>%1</b> has reached its maximum seed time of %2 hours and cannot be enqueued. "
 					"<br>Remove the limit manually if you want to continue seeding.",
-				s.torrent_name,loc->formatNumber(s.max_seed_time,2));
+				tc->getDisplayName(),loc->formatNumber(s.max_seed_time,2));
 		
 		KNotification::event("QueueNotPossible",msg,QPixmap(),mwnd);
 	}
@@ -253,7 +222,7 @@ namespace kt
 		if (!Settings::showPopups())
 			return;
 		
-		QString msg = i18n("Cannot start <b>%1</b> : <br>",tc->getStats().torrent_name);
+		QString msg = i18n("Cannot start <b>%1</b> : <br>",tc->getDisplayName());
 		switch (reason)
 		{
 		case bt::QM_LIMITS_REACHED:
@@ -285,9 +254,7 @@ namespace kt
 		if (!Settings::showPopups())
 			return;
 		
-		const TorrentStats & s = tc->getStats();
-		
-		QString msg = i18n("Your disk is running out of space.<br /><b>%1</b> is being downloaded to '%2'.",s.torrent_name,tc->getDataDir());
+		QString msg = i18n("Your disk is running out of space.<br /><b>%1</b> is being downloaded to '%2'.",tc->getDisplayName(),tc->getDataDir());
 		
 		if(stopped)
 			msg.prepend(i18n("Torrent has been stopped.<br />"));
@@ -323,7 +290,7 @@ namespace kt
 		int delta = 0;
 		int maxBandwidthRounded;
 
-		addTitle(i18n("Speed limit in KB/s"));
+		addTitle(i18n("Speed limit in KiB/s"));
 
 		unlimited = addAction(i18n("Unlimited"));
 		unlimited->setCheckable(true);
