@@ -70,6 +70,8 @@
 namespace bt
 {
 
+	
+
 	KUrl TorrentControl::completed_dir;
 	bool TorrentControl::completed_datacheck = false;
 	Uint32 TorrentControl::min_diskspace = 100;
@@ -93,7 +95,7 @@ namespace bt
 		istats.session_bytes_uploaded = 0;
 		old_tordir = QString();
 		stats.status = NOT_STARTED;
-		stats.autostart = true;
+		stats.autostart = false;
 		stats.user_controlled = false;
 		stats.priv_torrent = false;
 		stats.seeders_connected_to = stats.seeders_total = 0;
@@ -102,6 +104,7 @@ namespace bt
 		stats.max_share_ratio = 0.00f;
 		stats.max_seed_time = 0;
 		stats.last_download_activity_time = stats.last_upload_activity_time = 0;
+		stats.tracker_status = TRACKER_OK;
 		istats.running_time_dl = istats.running_time_ul = 0;
 		istats.prev_bytes_dl = 0;
 		istats.prev_bytes_ul = 0;
@@ -148,27 +151,14 @@ namespace bt
 	
 	bool TorrentControl::updateNeeded() const
 	{
-		return stats.running || moving_files || prealloc_thread || dcheck_thread;
+		return stats.running;
 	}
 
 	void TorrentControl::update()
 	{
 		UpdateCurrentTime();
-		if (moving_files)
+		if (moving_files || dcheck_thread || prealloc_thread)
 			return;
-
-		if (dcheck_thread)
-		{
-			if (!dcheck_thread->isRunning())
-			{
-				dcheck_thread->wait();
-				afterDataCheck();
-				if (!stats.running)
-					return;
-			}
-			else
-				return;
-		}
 		
 		if (istats.io_error)
 		{
@@ -177,17 +167,6 @@ namespace bt
 			return;
 		}
 		
-		if (prealloc_thread)
-		{
-			if (prealloc_thread->isDone())
-			{
-				prealloc_thread->wait();
-				preallocThreadDone();
-			}
-			else
-				return; // preallocation still going on, so just return
-		}
-		
 
 		try
 		{
@@ -195,6 +174,12 @@ namespace bt
 			pman->update();
 			bool comp = stats.completed;
 
+			// get rid of dead Peers
+			Uint32 num_cleared = pman->clearDeadPeers();
+			
+			// connect to new peers
+			pman->connectToPeers();
+
 			// then the downloader and uploader
 			uploader->update(choke->getOptimisticlyUnchokedPeerID());			
 			downloader->update();
@@ -209,7 +194,7 @@ namespace bt
 				pman->killSeeders();
 				QDateTime now = QDateTime::currentDateTime();
 				istats.running_time_dl += istats.time_started_dl.secsTo(now);
-				updateStatusMsg();
+				updateStatus();
 				updateStats();
 				
 				// download has just been completed
@@ -238,11 +223,24 @@ namespace bt
 					psman->manualUpdate();
 				istats.last_announce = bt::GetCurrentTime();
 				istats.time_started_dl = QDateTime::currentDateTime();
+				// Tell QM to redo queue
+				if (!isUserControlled())
+					updateQueue();
+			}
+			updateStatus();
+
+			if(wanted_update_timer.getElapsedSinceUpdate() >= 60*1000)
+			{
+				// Get a list of the chunks I have...
+				BitSet wanted_chunks = cman->getBitSet();
+				// or don't want...
+				wanted_chunks.orBitSet(cman->getExcludedBitSet());
+				wanted_chunks.orBitSet(cman->getOnlySeedBitSet());
+				// and inverted it get a list of the chunks I want
+				wanted_chunks.invert();
+				pman->setWantedChunks(wanted_chunks);
+				wanted_update_timer.update();
 			}
-			updateStatusMsg();
-			
-			// get rid of dead Peers
-			Uint32 num_cleared = pman->clearDeadPeers();
 			
 			// we may need to update the choker
 			if (choker_update_timer.getElapsedSinceUpdate() >= 10000 || num_cleared > 0)
@@ -292,15 +290,14 @@ namespace bt
 			
 			if (overMaxRatio() || overMaxSeedTime()) 
 			{ 
-				if(istats.priority!=0) //if it's queued make sure to dequeue it 
+				if (!stats.user_controlled) //if it's queued make sure to dequeue it 
 				{
-					setPriority(0);
-					stats.user_controlled = true;
+					setUserControlled(true);
 				}
                  
 				stop(true); 
 				emit seedingAutoStopped(this, overMaxRatio() ? MAX_RATIO_REACHED : MAX_SEED_TIME_REACHED);
-			} 			
+			}
 
 			//Update diskspace if needed (every 1 min)			
 			if(!stats.completed && stats.running && bt::GetCurrentTime() - last_diskspace_check >= 60 * 1000)
@@ -313,13 +310,17 @@ namespace bt
 				needDataCheck(this);
 			
 			//Move completed files if needed:
-			if(moveCompleted)
+			if (moveCompleted)
 			{
-				QString outdir = completed_dir.path();
-				if (!outdir.endsWith(bt::DirSeparator()))
-					outdir += bt::DirSeparator();
-				
-				changeOutputDir(outdir,bt::TorrentInterface::MOVE_FILES);
+				if (stats.status == CHECKING_DATA)
+				{
+					// wait for dcheck_thread to finish before moving the files
+					connect(this,SIGNAL(dataCheckFinished()),this,SLOT(moveToCompletedDir()));
+				}
+				else
+				{
+					moveToCompletedDir();
+				}
 			}
 		}
 		catch (Error & e)
@@ -381,6 +382,7 @@ namespace bt
 			{
 				Out(SYS_GEN|LOG_NOTICE) << "Pre-allocating diskspace" << endl;
 				prealloc_thread = new PreallocationThread(cman);
+				connect(prealloc_thread,SIGNAL(finished()),this,SLOT(preallocThreadDone()),Qt::QueuedConnection);
 				stats.running = true;
 				stats.status = ALLOCATING_DISKSPACE;
 				prealloc_thread->start();
@@ -419,7 +421,7 @@ namespace bt
 		stats.last_download_activity_time = stats.last_upload_activity_time = GetCurrentTime();
 		choker_update_timer.update();
 		stats_save_timer.update();
-		
+		wanted_update_timer.update();
 		
 		stalled_timer.update();
 		psman->start();
@@ -439,22 +441,13 @@ namespace bt
 		// stop preallocation thread if necesarry
 		if (prealloc_thread)
 		{
+			disconnect(prealloc_thread,SIGNAL(finished()),this,SLOT(preallocThreadDone()));
 			prealloc_thread->stop();
 			prealloc_thread->wait();
-			
 			if (prealloc_thread->errorHappened() || prealloc_thread->isNotFinished())
-			{
-				delete prealloc_thread;
-				prealloc_thread = 0;
-				prealloc = true;
 				saveStats(); // save stats, so that we will start preallocating the next time
-			}
-			else
-			{
-				delete prealloc_thread;
-				prealloc_thread = 0;
-				prealloc = false;
-			}
+			prealloc_thread->deleteLater();
+			prealloc_thread = 0;
 		}
 	
 		if (stats.running)
@@ -480,8 +473,8 @@ namespace bt
 		
 		if (user)
 		{
-				//make this torrent user controlled
-			setPriority(0);
+			//make this torrent user controlled
+			setUserControlled(true);
 			stats.autostart = false;
 		}
 		
@@ -493,7 +486,7 @@ namespace bt
 		
 		stats.running = false;
 		saveStats();
-		updateStatusMsg();
+		updateStatus();
 		updateStats();
 		stats.trk_bytes_downloaded = 0;
 		stats.trk_bytes_uploaded = 0;
@@ -510,9 +503,12 @@ namespace bt
 			for (Uint32 i = 0;i < pman->getNumConnectedPeers();i++)
 				tmon->peerAdded(pman->getPeer(i));
 		}
+		
+		tor->setMonitor(tmon);
 	}
 	
 	
+	
 	void TorrentControl::init(QueueManagerInterface* qman,
 							  const QString & torrent,
 							  const QString & tmpdir,
@@ -529,7 +525,7 @@ namespace bt
 		{
 			delete tor;
 			tor = 0;
-			throw Error(i18n("An error occurred while loading the torrent <b>%1</b>.<br/>"
+			throw Error(i18n("An error occurred while loading the torrent <b>%1</b>\n"
 					"The torrent is probably corrupt or is not a torrent file.",torrent));
 		}
 		
@@ -541,7 +537,6 @@ namespace bt
 		{
 			bt::CopyFile(torrent,tor_copy);
 		}
-		
 	}
 	
 	
@@ -558,7 +553,7 @@ namespace bt
 		{
 			delete tor;
 			tor = 0;
-			throw Error(i18n("An error occurred while loading the torrent.<br/>"
+			throw Error(i18n("An error occurred while loading the torrent.\n"
 					"The torrent is probably corrupt or is not a torrent file."));
 		}
 		
@@ -629,35 +624,33 @@ namespace bt
 			loadOutputDir();
 	}
 	
-	void TorrentControl::setupData(const QString & ddir)
+	void TorrentControl::setupData()
 	{
 		// create PeerManager and Tracker
 		pman = new PeerManager(*tor);
 		//Out() << "Tracker url " << url << " " << url.protocol() << " " << url.prettyURL() << endl;
 		psman = new PeerSourceManager(this,pman);
-		connect(psman,SIGNAL(statusChanged( const QString& )),
-				this,SLOT(trackerStatusChanged( const QString& )));
-
+		connect(psman,SIGNAL(statusChanged(TrackerStatus , const QString& )),
+				this,SLOT(trackerStatusChanged(TrackerStatus , const QString& )));
 
 		// Create chunkmanager, load the index file if it exists
 		// else create all the necesarry files
 		cman = new ChunkManager(*tor,tordir,outputdir,istats.custom_output_name,cache_factory);
-		
-		connect(cman,SIGNAL(updateStats()),this,SLOT(updateStats()));
 		if (bt::Exists(tordir + "index"))
 			cman->loadIndexFile();
-
+		
+		connect(cman,SIGNAL(updateStats()),this,SLOT(updateStats()));
+		updateStats();
 		stats.completed = cman->completed();
 
 		// create downloader,uploader and choker
 		downloader = new Downloader(*tor,*pman,*cman,custom_selector_factory);
 		downloader->loadWebSeeds(tordir + "webseeds");
-		connect(downloader,SIGNAL(ioError(const QString& )),
-				this,SLOT(onIOError(const QString& )));
+		connect(downloader,SIGNAL(ioError(const QString& )),this,SLOT(onIOError(const QString& )));
+		connect(downloader,SIGNAL(chunkDownloaded(Uint32)),this,SLOT(downloaded(Uint32)));
 		uploader = new Uploader(*cman,*pman);
 		choke = new Choker(*pman,*cman);
 
-
 		connect(pman,SIGNAL(newPeer(Peer* )),this,SLOT(onNewPeer(Peer* )));
 		connect(pman,SIGNAL(peerKilled(Peer* )),this,SLOT(onPeerRemoved(Peer* )));
 		connect(cman,SIGNAL(excluded(Uint32, Uint32 )),downloader,SLOT(onExcluded(Uint32, Uint32 )));
@@ -674,6 +667,7 @@ namespace bt
 		checkExisting(qman);
 		setupDirs(tmpdir,ddir);
 		setupStats();
+		loadEncoding();
 
 		if (!first_time)
 		{
@@ -689,10 +683,9 @@ namespace bt
 				throw Error(i18n("Cannot migrate %1 : %2",tor->getNameSuggestion(),err.toString()));
 			}
 		}
-		setupData(ddir);
-
-		updateStatusMsg();
-
+		setupData();
+		updateStatus();
+		
 		// to get rid of phantom bytes we need to take into account
 		// the data from downloads already in progress
 		try
@@ -715,15 +708,14 @@ namespace bt
 		updateStats();
 		saveStats();
 		stats.output_path = cman->getOutputPath();
-	/*	if (stats.output_path.isNull())
-		{
-			cman->createFiles();
-			stats.output_path = cman->getOutputPath();
-		}*/
-		Out(SYS_GEN|LOG_DEBUG) << "OutputPath = " << stats.output_path << endl;
+		updateStatus();
 	}
 	
-
+	void TorrentControl::setDisplayName(const QString & n)
+	{
+		display_name = n;
+		saveStats();
+	}
 
 	bool TorrentControl::announceAllowed()
 	{
@@ -982,14 +974,16 @@ namespace bt
 		}
 	}
 
-	void TorrentControl::updateStatusMsg()
+	void TorrentControl::updateStatus()
 	{
 		TorrentStatus old = stats.status;
 		if (stats.stopped_by_error)
 			stats.status = ERROR;
-		else if (!stats.started)
+		else if (dcheck_thread)
+			stats.status = CHECKING_DATA;
+		else if (!stats.started && stats.user_controlled)
 			stats.status = NOT_STARTED;
-		else if(!stats.running && !stats.user_controlled)
+		else if (!stats.running && !stats.user_controlled)
 			stats.status = QUEUED;
 		else if (!stats.running && stats.completed && (overMaxRatio() || overMaxSeedTime()))
 			stats.status = SEEDING_COMPLETE;
@@ -1064,6 +1058,7 @@ namespace bt
 		}
 		
 		st.write("PRIORITY", QString("%1").arg(istats.priority));
+		st.write("USER_CONTROLLED",stats.user_controlled ? "1" : "0");
 		st.write("AUTOSTART", QString("%1").arg(stats.autostart));
 		st.write("IMPORTED", QString("%1").arg(stats.imported_bytes));
 		st.write("CUSTOM_OUTPUT_NAME",istats.custom_output_name ? "1" : "0");
@@ -1083,11 +1078,22 @@ namespace bt
 		st.write("ENCODING",QString(tor->getTextCodec()->name()));
 		st.write("ASSURED_UPLOAD_SPEED",QString::number(assured_upload_speed));
 		st.write("ASSURED_DOWNLOAD_SPEED",QString::number(assured_download_speed));
+		if (!user_modified_name.isEmpty())
+			st.write("USER_MODIFIED_NAME",user_modified_name);
+		st.write("DISPLAY_NAME",display_name);
+		st.write("URL",url.prettyUrl());
 		st.writeSync();
 	}
 
 	void TorrentControl::loadStats()
 	{
+		if (!bt::Exists(tordir + "stats"))
+		{
+			setFeatureEnabled(DHT_FEATURE,true);
+			setFeatureEnabled(UT_PEX_FEATURE,true);
+			return;
+		}
+		
 		StatsFile st(tordir + "stats");
 		
 		Uint64 val = st.readUint64("UPLOADED");
@@ -1110,8 +1116,18 @@ namespace bt
 			istats.custom_output_name = true;
 		}
 		
+		if (st.hasKey("USER_MODIFIED_NAME"))
+			user_modified_name = st.readString("USER_MODIFIED_NAME");
+		
+		if (st.hasKey("DISPLAY_NAME"))
+			display_name = st.readString("DISPLAY_NAME");
+		
 		setPriority(st.readInt("PRIORITY"));
-		stats.user_controlled = istats.priority == 0 ? true : false;
+		// before USER_CONTROLLED was added, priority == 0 meant user controlled
+		if (st.hasKey("USER_CONTROLLED"))
+			setUserControlled(st.readBoolean("USER_CONTROLLED"));
+		else
+			setUserControlled(getPriority() == 0); 
 		stats.autostart = st.readBoolean("AUTOSTART");
 		
 		stats.imported_bytes = st.readUint64("IMPORTED");
@@ -1149,6 +1165,9 @@ namespace bt
 		setDownloadProps(down,adown);
 		setUploadProps(up,aup);
 		pman->setGroupIDs(upload_gid,download_gid);
+		
+		if (!url.isValid())
+			url = KUrl(st.readString("URL"));
 	}
 
 	void TorrentControl::loadOutputDir()
@@ -1163,6 +1182,21 @@ namespace bt
 			istats.custom_output_name = true;
 		}
 	}
+	
+	void TorrentControl::loadEncoding()
+	{
+		StatsFile st(tordir + "stats");
+		if (!st.hasKey("ENCODING"))
+			return;
+		
+		QString codec = st.readString("ENCODING");
+		if (codec.length() > 0)
+		{
+			QTextCodec* cod = QTextCodec::codecForName(codec.toLocal8Bit());
+			if (cod)
+				changeTextCodec(cod);
+		}
+	}
 
 	bool TorrentControl::readyForPreview() const
 	{
@@ -1371,16 +1405,15 @@ namespace bt
 	void TorrentControl::setPriority(int p)
 	{
 		istats.priority = p;
-		stats.user_controlled = p == 0 ? true : false;
-		if(p)
-		{
-			stats.status = QUEUED;
-			statusChanged(this);
-		}
-		else
-			updateStatusMsg();
-	
 		saveStats();
+		updateStatus();
+	}
+	
+	void TorrentControl::setUserControlled(bool uc)
+	{
+		stats.user_controlled = uc;
+		saveStats();
+		updateStatus();
 	}
 	
 	void TorrentControl::setMaxShareRatio(float ratio)
@@ -1394,7 +1427,7 @@ namespace bt
 			stats.max_share_ratio = ratio;
 		
 		if(stats.completed && !stats.running && !stats.user_controlled && (ShareRatio(stats) >= stats.max_share_ratio))
-			setPriority(0); //dequeue it
+			setUserControlled(true); //dequeue it
 		
 		saveStats();
 		emit maxRatioChanged(this);
@@ -1483,7 +1516,6 @@ namespace bt
 		if (stats.status == ALLOCATING_DISKSPACE)
 			return;
 		
-		
 		DataChecker* dc = 0;
 		stats.status = CHECKING_DATA;
 		stats.num_corrupted_chunks = 0; // reset the number of corrupted chunks found
@@ -1495,9 +1527,10 @@ namespace bt
 		dc->setListener(lst);
 		
 		dcheck_thread = new DataCheckerThread(dc,cman->getBitSet(),stats.output_path,*tor,tordir + "dnd" + bt::DirSeparator());
+		connect(dcheck_thread,SIGNAL(finished()),this,SLOT(afterDataCheck()),Qt::QueuedConnection);
 		
 		// dc->check(stats.output_path,*tor,tordir + "dnd" + bt::DirSeparator());
-		dcheck_thread->start();
+		dcheck_thread->start(QThread::IdlePriority);
 		statusChanged(this);
 	}
 	
@@ -1514,6 +1547,7 @@ namespace bt
 			lst->stop();
 		}
 		
+		bool completed = stats.completed;
 		if (lst && !lst->isStopped())
 		{
 			downloader->dataChecked(dc->getResult());
@@ -1523,8 +1557,7 @@ namespace bt
 			{
 				downloader->recalcDownloaded();
 				stats.imported_bytes = downloader->bytesDownloaded();
-				if (cman->haveAllChunks())
-					stats.completed = true;
+				stats.completed = cman->completed();
 			}
 			else
 			{
@@ -1534,19 +1567,28 @@ namespace bt
 				if (stats.bytes_downloaded > downloaded)
 					stats.imported_bytes = stats.bytes_downloaded - downloaded;
 				 
-				if (cman->haveAllChunks())
-					stats.completed = true;
+				stats.completed = cman->completed();
 			}
 		}
 			
-		stats.status = NOT_STARTED;
-		// update the status
-		updateStatusMsg();
 		updateStats();
+		dcheck_thread->deleteLater();
+		dcheck_thread = 0;
+		Out(SYS_GEN|LOG_NOTICE) << "Data check finished" << endl;
+		resetTrackerStats();
+		updateStatus();
 		if (lst)
 			lst->finished();
-		delete dcheck_thread;
-		dcheck_thread = 0;
+		
+		dataCheckFinished();
+	
+		if (stats.completed != completed)
+		{
+			// Tell QM to redo queue 
+			// seeder might have become downloader, so 
+			// queue might need to be redone
+			updateQueue();
+		}
 	}
 	
 	bool TorrentControl::isCheckingData(bool & finished) const
@@ -1644,9 +1686,10 @@ namespace bt
 		stats.trk_bytes_uploaded = 0;
 	}
 	
-	void TorrentControl::trackerStatusChanged(const QString & ns)
+	void TorrentControl::trackerStatusChanged(TrackerStatus s,const QString & ns)
 	{
-		stats.trackerstatus = ns;
+		stats.tracker_status = s;
+		stats.tracker_status_string = ns;
 	}
 	
 	void TorrentControl::addPeerSource(PeerSource* ps)
@@ -1706,7 +1749,7 @@ namespace bt
 		case DHT_FEATURE:
 			if (on)
 			{
-				if(!stats.priv_torrent)
+				if (!stats.priv_torrent)
 				{
 					psman->addDHT();
 					istats.dht_on = psman->dhtStarted();
@@ -1885,19 +1928,22 @@ namespace bt
 	
 	void TorrentControl::preallocThreadDone()
 	{
+		if (!prealloc_thread)
+			return;
+		
 		// thread done
 		if (prealloc_thread->errorHappened())
 		{
 			// upon error just call onIOError and return
 			onIOError(prealloc_thread->errorMessage());
-			delete prealloc_thread;
+			prealloc_thread->deleteLater();
 			prealloc_thread = 0;
 			prealloc = true; // still need to do preallocation
 		}
 		else
 		{
 			// continue the startup of the torrent
-			delete prealloc_thread;
+			prealloc_thread->deleteLater();
 			prealloc_thread = 0;
 			prealloc = false;
 			stats.status = NOT_STARTED;
@@ -1962,6 +2008,39 @@ namespace bt
 			downloader->saveWebSeeds(tordir + "webseeds");
 		return ret;
 	}
+	
+	void TorrentControl::setUserModifiedFileName(const QString & n)
+	{
+		TorrentInterface::setUserModifiedFileName(n);
+		QString path = getDataDir();
+		if (!path.endsWith(bt::DirSeparator()))
+			path += bt::DirSeparator();
+		
+		cman->changeOutputPath(path + n);
+		outputdir = stats.output_path = path + n;
+		istats.custom_output_name = true;
+	}
+	
+	void TorrentControl::downloaded(Uint32 chunk)
+	{
+		chunkDownloaded(this,chunk);
+	}
+	
+	void TorrentControl::moveToCompletedDir()
+	{
+		disconnect(this,SIGNAL(dataCheckFinished()),this,SLOT(moveToCompletedDir()));
+		
+		// it is possible that the user might have disabled moving to the completed dir during the data check
+		// so double check before we start the move
+		if (completed_dir.path().isNull() || !stats.completed)
+			return;
+			
+		QString outdir = completed_dir.path();
+		if (!outdir.endsWith(bt::DirSeparator()))
+			outdir += bt::DirSeparator();
+					
+		changeOutputDir(outdir,bt::TorrentInterface::MOVE_FILES);
+	}
 }
 
 #include "torrentcontrol.moc"
