@@ -28,6 +28,7 @@
 #include <util/sha1hash.h>
 #include <util/waitjob.h>
 #include <util/fileops.h>
+#include <util/functions.h>
 #include <torrent/globals.h>
 #include <torrent/torrent.h>
 #include <torrent/torrentcontrol.h>
@@ -55,7 +56,9 @@ namespace kt
 
 
 	QueueManager::~QueueManager()
-	{}
+	{
+		qDeleteAll(downloads);
+	}
 
 	void QueueManager::append(bt::TorrentInterface* tc)
 	{
@@ -63,6 +66,7 @@ namespace kt
 		downloads.sort();
 		connect(tc, SIGNAL(diskSpaceLow(bt::TorrentInterface*, bool)), this, SLOT(onLowDiskSpace(bt::TorrentInterface*, bool)));	
 		connect(tc, SIGNAL(torrentStopped(bt::TorrentInterface*)), this, SLOT(torrentStopped(bt::TorrentInterface*)));
+		connect(tc,SIGNAL(updateQueue()),this,SLOT(orderQueue()));
 	}
 
 	void QueueManager::remove(bt::TorrentInterface* tc)
@@ -76,12 +80,14 @@ namespace kt
 	void QueueManager::clear()
 	{
 		Uint32 nd = downloads.count();
-		downloads.clear();
 		paused_torrents.clear();
 		
 		// wait for a second to allow all http jobs to send the stopped event
 		if (nd > 0)
 			SynchronousWait(1000);
+		
+		qDeleteAll(downloads);
+		downloads.clear();
 	}
 	
 	TorrentStartResponse QueueManager::startInternal(bt::TorrentInterface* tc,bool user)
@@ -94,12 +100,12 @@ namespace kt
 			switch (Settings::startDownloadsOnLowDiskSpace())
 			{
 			case 0: //don't start!
-				tc->setPriority(0);
+				tc->setUserControlled(true);
 				return bt::NOT_ENOUGH_DISKSPACE;
 			case 1: //ask user
 				if (KMessageBox::questionYesNo(0, i18n("You don't have enough disk space to download this torrent. Are you sure you want to continue?"), i18n("Insufficient disk space for %1",s.torrent_name)) == KMessageBox::No)
 				{
-					tc->setPriority(0);
+					tc->setUserControlled(true);
 					return bt::USER_CANCELED;
 				}
 				else
@@ -116,7 +122,7 @@ namespace kt
 		{
 			if (!user)
 			{
-				tc->setPriority(0); // dequeue torrent
+				tc->setUserControlled(true); // dequeue torrent
 				return QM_LIMITS_REACHED;
 			}
 			
@@ -164,7 +170,7 @@ namespace kt
 		else
 		{
 			//User started this torrent so make it user controlled
-			tc->setPriority(0);
+			tc->setUserControlled(true);
 		}
 
 		if (start_tc)
@@ -192,7 +198,7 @@ namespace kt
 		}
 		
 		if (user) //dequeue it
-			tc->setPriority(0);
+			tc->setUserControlled(true);
 	}
 	
 	void QueueManager::checkDiskSpace(QList<bt::TorrentInterface*> & todo)
@@ -324,7 +330,7 @@ namespace kt
 			if (tc->isCheckingData(check_done) && !check_done)
 				continue;
 			
-			tc->setPriority(0);
+			tc->setUserControlled(true);
 			startSafely(tc);
 		}
 	}
@@ -354,6 +360,7 @@ namespace kt
 
 	void QueueManager::stopall(int type)
 	{
+		ordering = true;
 		QList<bt::TorrentInterface *>::iterator i = downloads.begin();
 		while (i != downloads.end())
 		{
@@ -376,11 +383,12 @@ namespace kt
 					KMessageBox::error(0,msg,i18n("Error"));
 				}
 			}
-			else //if torrent is not running but it is queued we need to make it user controlled
-				if( (s.completed && type == 2) || (!s.completed && type == 1) || (type == 3) )
-					tc->setPriority(0); 
+			//if torrent is not running but it is queued we need to make it user controlled
+			else if( (s.completed && type == 2) || (!s.completed && type == 1) || (type == 3) )
+					tc->setUserControlled(true);
 			i++;
 		}
+		ordering = false;
 	}
 	
 	void QueueManager::onExit(WaitJob* wjob)
@@ -416,8 +424,8 @@ namespace kt
 	int QueueManager::getNumRunning(Flags flags)
 	{
 		int nr = 0;
-		QList<TorrentInterface*>::const_iterator i = downloads.begin();
-		while (i != downloads.end())
+		QList<TorrentInterface*>::const_iterator i = downloads.constBegin();
+		while (i != downloads.constEnd())
 		{
 			const TorrentInterface* tc = *i;
 			const TorrentStats & s = tc->getStats();
@@ -509,25 +517,24 @@ namespace kt
 	
 	void QueueManager::orderQueue()
 	{
+		downloads.sort(); // sort downloads, even when paused so that the QM widget is updated
 		if (ordering || !downloads.count() || paused_state || exiting)
 			return;
 		
 		ordering = true; // make sure that recursive entering of this function is not possible
 		
-		downloads.sort();
-	
 		QueuePtrList download_queue;
 		QueuePtrList seed_queue;
 		
 		foreach (TorrentInterface* tc,downloads)
 		{
 			const TorrentStats & s = tc->getStats();
-			
-			if (!s.user_controlled && !tc->isMovingFiles() && !s.stopped_by_error)
+			bool dummy;
+			if (!s.user_controlled && !tc->isMovingFiles() && !s.stopped_by_error && !tc->isCheckingData(dummy))
 			{
 				if (s.completed)
 					seed_queue.append(tc);
-				else		
+				else
 					download_queue.append(tc);
 			}
 		}
@@ -545,7 +552,7 @@ namespace kt
 					if (startInternal(tc,false) == bt::START_OK)
 						num_running++;
 					else if (s.stopped_by_error)
-						tc->setPriority(0);
+						tc->setUserControlled(true);
 				}
 				else
 					num_running++;
@@ -557,6 +564,8 @@ namespace kt
 					Out(SYS_GEN|LOG_DEBUG) << "QM Stopping: " << s.torrent_name << endl;
 					stop(tc);
 				}
+				else
+					tc->updateStatus();
 			}
 		}
 		
@@ -573,7 +582,7 @@ namespace kt
 					if (startInternal(tc,false) == bt::START_OK)
 						num_running++;
 					else if (s.stopped_by_error)
-						tc->setPriority(0);
+						tc->setUserControlled(true);
 				}
 				else
 					num_running++;
@@ -585,6 +594,8 @@ namespace kt
 					Out(SYS_GEN|LOG_DEBUG) << "QM Stopping: " << s.torrent_name << endl;
 					stop(tc);
 				}
+				else
+					tc->updateStatus();
 			}
 		}
 		
@@ -594,15 +605,7 @@ namespace kt
 	
 	void QueueManager::torrentFinished(bt::TorrentInterface* tc)
 	{
-		//dequeue this tc
-		tc->setPriority(0);
-		//make sure the max_seeds is not reached
-// 		if(max_seeds !=0 && max_seeds < getNumRunning(false,true))
-// 			tc->stop(true);
-		
-		if (keep_seeding)
-			torrentAdded(tc,false,false);
-		else
+		if (!keep_seeding)
 			stopSafely(tc,true);
 		
 		orderQueue();
@@ -612,20 +615,21 @@ namespace kt
 	{
 		if (!user)
 		{
+			// new torrents have the lowest priority
+			// so everybody else gets a higher priority
 			foreach (TorrentInterface * otc,downloads)
 			{
 				int p = otc->getPriority();
-				if (p > 0)
-					otc->setPriority(p+1);
+				otc->setPriority(p+1);
 			}
-			tc->setPriority(1);
+			tc->setPriority(0);
 			rearrangeQueue();
 			orderQueue();
 		}
 		else
 		{
-			 tc->setPriority(0);
-			 if(start_torrent)
+			 tc->setUserControlled(true);
+			 if (start_torrent)
 				 start(tc, true);
 		}	
 	}
@@ -639,9 +643,13 @@ namespace kt
 	
 	void QueueManager::setPausedState(bool pause)
 	{
+		if (paused_state == pause)
+			return;
+		
 		paused_state = pause;	
 		if(!pause)
 		{
+			UpdateCurrentTime();
 			std::set<bt::TorrentInterface*>::iterator it = paused_torrents.begin();
 			while (it != paused_torrents.end())
 			{
@@ -675,8 +683,7 @@ namespace kt
 		// make sure everybody has an unique priority
 		foreach (bt::TorrentInterface* tc,downloads)
 		{
-			if (tc->getPriority() > 0)
-				tc->setPriority(prio--);
+			tc->setPriority(prio--);
 		}
 	}
 	
@@ -690,19 +697,19 @@ namespace kt
 			return;
 		}
 		
-		torrentAdded(tc,false,false);
+		tc->setUserControlled(false);
+		orderQueue();
 	}
 	
 	void QueueManager::dequeue(bt::TorrentInterface* tc)
 	{
-		tc->setPriority(0);
-		rearrangeQueue();
+		tc->setUserControlled(true);
 		orderQueue();
 	}
 	
 	void QueueManager::queue(bt::TorrentInterface* tc)
 	{
-		if(tc->getPriority() == 0)
+		if (tc->isUserControlled())
 			enqueue(tc);
 		else
 			dequeue(tc);
@@ -758,34 +765,46 @@ namespace kt
 	
 	void QueueManager::checkStalledTorrents(bt::TimeStamp now,bt::Uint32 min_stall_time)
 	{
-		const int MAX_PRIO = INT_MAX;
-		bool found_stalled = false;
-		// set the priority of all stalled ones to -1
+		QueuePtrList newlist;
+		QueuePtrList stalled;
+		bool can_decrease = false;
+		
+		// find all stalled ones
 		foreach (bt::TorrentInterface* tc,downloads)
 		{
-			if (tc->getPriority() > 1 && IsStalled(tc,now,min_stall_time))
+			if (!tc->isUserControlled())
 			{
-				tc->setPriority(MAX_PRIO);
-				found_stalled = true;
-				Out(SYS_GEN | LOG_NOTICE) << "The torrent " << tc->getStats().torrent_name << " has stalled longer then " << min_stall_time << " minutes, decreasing it's priority" << endl;
+				if (IsStalled(tc,now,min_stall_time))
+				{
+					stalled.append(tc);
+				}
+				else
+				{
+					// decreasing makes only sense if there are QM torrents after the stalled ones
+					can_decrease = stalled.count() > 0;
+					newlist.append(tc);
+				}
 			}
+			else 
+				newlist.append(tc);
 		}
+		
+		if (stalled.count() == 0 || stalled.count() == downloads.count() || !can_decrease)
+			return;
+		
+		foreach (bt::TorrentInterface* tc,stalled)
+			Out(SYS_GEN | LOG_NOTICE) << "The torrent " << tc->getStats().torrent_name << " has stalled longer then " << min_stall_time << " minutes, decreasing it's priority" << endl;
 	
-		if (found_stalled)
+		downloads.clear();
+		downloads += newlist;
+		downloads += stalled;
+		// redo priorities and then order the queue
+		int prio = downloads.count();
+		foreach (bt::TorrentInterface* tc,downloads)
 		{
-			// redo the priority, stalled ones now get 1, 
-			// the other ones get an increase if they are not user controlled
-			foreach (bt::TorrentInterface* tc,downloads)
-			{
-				int prio = tc->getPriority();
-				if (prio == MAX_PRIO)
-					tc->setPriority(1);
-				else if (prio > 1 || (prio == 1 && !IsStalled(tc,now,min_stall_time)))
-					tc->setPriority(prio + 1);
-			}
-			
-			orderQueue();
+			tc->setPriority(prio--);
 		}
+		orderQueue();
 	}
 	/////////////////////////////////////////////////////////////////////////////////////////////
 
